    <script async src="https://www.googletagmanager.com/gtag/js?id=default-key"></script>
    <script>
      const google = {"enabled":false,"key":"default-key","sampleRate":100};
      window.dataLayer = window.dataLayer || [];
      function gtag() { dataLayer.push(arguments); }
      gtag('js', new Date());
      gtag('config', google.key);
      gtag('create', google.key, { 'siteSpeedSampleRate': google.sampleRate });
    </script>
